@@ -2,29 +2,21 @@ import os
 import copy
 import uuid
 import atexit
-import threading
 import logging
 import types
 import time
-from yggdrasil.tests import assert_equal
-from yggdrasil import tools
+from yggdrasil import tools, multitasking
 from yggdrasil.tools import YGG_MSG_EOF
 from yggdrasil.communication import new_comm, get_comm, determine_suffix
 from yggdrasil.components import import_component, create_component
 from yggdrasil.metaschema.datatypes import MetaschemaTypeError
 from yggdrasil.metaschema.datatypes.MetaschemaType import MetaschemaType
-from yggdrasil.metaschema.datatypes.JSONArrayMetaschemaType import (
-    JSONArrayMetaschemaType)
-from yggdrasil.metaschema.datatypes.JSONObjectMetaschemaType import (
-    JSONObjectMetaschemaType)
 from yggdrasil.communication.transforms.TransformBase import TransformBase
 
 
 logger = logging.getLogger(__name__)
-_registered_servers = dict()
-_registered_comms = dict()
-_server_lock = threading.RLock()
-_registry_lock = threading.RLock()
+_registered_servers = multitasking.LockedDict(task_method='thread')
+_registered_comms = multitasking.LockedDict(task_method='thread')
 
 
 def is_registered(comm_class, key):
@@ -35,8 +27,8 @@ def is_registered(comm_class, key):
         key (str): Key that should be checked.
 
     """
-    with _registry_lock:
-        global _registered_comms
+    global _registered_comms
+    with _registered_comms.lock:
         if comm_class not in _registered_comms:
             return False
         return (key in _registered_comms[comm_class])
@@ -52,7 +44,7 @@ def get_comm_registry(comm_class):
         dict: Dictionary of registered comm objects.
 
     """
-    with _registry_lock:
+    with _registered_comms.lock:
         if comm_class is None:
             out = {}
         else:
@@ -69,10 +61,10 @@ def register_comm(comm_class, key, value):
         value (obj): Object being registered.
 
     """
-    with _registry_lock:
-        global _registered_comms
+    global _registered_comms
+    with _registered_comms.lock:
         if comm_class not in _registered_comms:
-            _registered_comms[comm_class] = dict()
+            _registered_comms.add_subdict(comm_class)
         if key not in _registered_comms[comm_class]:
             _registered_comms[comm_class][key] = value
 
@@ -90,8 +82,8 @@ def unregister_comm(comm_class, key, dont_close=False):
         bool: True if an object was closed.
 
     """
-    with _registry_lock:
-        global _registered_comms
+    global _registered_comms
+    with _registered_comms.lock:
         if comm_class not in _registered_comms:
             return False
         if key not in _registered_comms[comm_class]:
@@ -117,10 +109,10 @@ def cleanup_comms(comm_class, close_func=None):
     count = 0
     if comm_class is None:
         return count
-    with _registry_lock:
-        global _registered_comms
+    global _registered_comms
+    with _registered_comms.lock:
         if comm_class in _registered_comms:
-            keys = [k for k in _registered_comms[comm_class].keys()]
+            keys = list(_registered_comms[comm_class].keys())
             for k in keys:
                 flag = unregister_comm(comm_class, k)
                 if flag:  # pragma: debug
@@ -128,26 +120,26 @@ def cleanup_comms(comm_class, close_func=None):
     return count
 
 
-class CommThreadLoop(tools.YggThreadLoop):
-    r"""Thread loop for comms to ensure cleanup.
+class CommTaskLoop(multitasking.YggTaskLoop):
+    r"""Task loop for comms to ensure cleanup.
 
     Args:
         comm (:class:.CommBase): Comm class that thread is for.
         name (str, optional): Name for the thread. If not provided, one is
             created by combining the comm name and the provided suffix.
         suffix (str, optional): Suffix that should be added to comm name to name
-            the thread. Defaults to 'CommThread'.
+            the thread. Defaults to 'CommTask'.
         **kwargs: Additional keyword arguments are passed to the parent class.
 
     Attributes:
         comm (:class:.CommBase): Comm class that thread is for.
 
     """
-    def __init__(self, comm, name=None, suffix='CommThread', **kwargs):
+    def __init__(self, comm, name=None, suffix='CommTask', **kwargs):
         self.comm = comm
         if name is None:
             name = '%s.%s' % (comm.name, suffix)
-        super(CommThreadLoop, self).__init__(name=name, **kwargs)
+        super(CommTaskLoop, self).__init__(name=name, **kwargs)
 
     def on_main_terminated(self):  # pragma: debug
         r"""Actions taken on the backlog thread when the main thread stops."""
@@ -162,10 +154,10 @@ class CommThreadLoop(tools.YggThreadLoop):
                 self.debug("Close in thread, closed = %s, nmsg = %d",
                            self.comm.is_closed, self.comm.n_msg)
                 return
-        super(CommThreadLoop, self).on_main_terminated()
+        super(CommTaskLoop, self).on_main_terminated()
 
 
-class CommServer(tools.YggThreadLoop):
+class CommServer(multitasking.YggTaskLoop):
     r"""Basic server object to keep track of clients.
 
     Attributes:
@@ -296,6 +288,11 @@ class CommBase(tools.YggClass):
             input/outputs to/from a model being wrapped. The receive/send
             calls for this comm will be outside the loop for the model.
             Defaults to False.
+        default_file (:class:FileComm, optional): Comm information for
+            a file that input should be drawn from (for input comms)
+            or that output should be sent to (for output comms) in
+            the event that a yaml does not pair the comm with another
+            model comm or a file.
         **kwargs: Additional keywords arguments are passed to parent class.
 
     Class Attributes:
@@ -400,6 +397,9 @@ class CommBase(tools.YggClass):
     address_description = None
     no_serialization = False
     _model_schema_prop = ['is_default', 'outside_loop', 'default_file']
+    _disconnect_attr = (tools.YggClass._disconnect_attr
+                        + ['_closing_event', '_closing_thread',
+                           '_eof_recv', '_eof_sent'])
 
     def __init__(self, name, address=None, direction='send', dont_open=False,
                  is_interface=None, language=None, partner_language='python',
@@ -407,14 +407,14 @@ class CommBase(tools.YggClass):
                  single_use=False, reverse_names=False, no_suffix=False,
                  is_client=False, is_response_client=False,
                  is_server=False, is_response_server=False,
-                 comm=None, **kwargs):
+                 comm=None, touches_model=False, **kwargs):
         self._comm_class = None
         if comm is not None:
             assert(comm == self.comm_class)
         if isinstance(kwargs.get('datatype', None), MetaschemaType):
             self.datatype = kwargs.pop('datatype')
         super(CommBase, self).__init__(name, **kwargs)
-        if not self.__class__.is_installed(language='python'):
+        if not self.__class__.is_installed(language='python'):  # pragma: debug
             raise RuntimeError("Comm class %s not installed" % self.__class__)
         suffix = determine_suffix(no_suffix=no_suffix,
                                   reverse_names=reverse_names,
@@ -424,7 +424,7 @@ class CommBase(tools.YggClass):
         self._name = name + suffix
         if address is None:
             if self.name not in os.environ:
-                model_name = os.environ.get('YGG_MODEL_NAME', '')
+                model_name = self.model_name
                 prefix = '%s:' % model_name
                 if model_name and (not self.name.startswith(prefix)):
                     self._name = prefix + self.name
@@ -461,6 +461,7 @@ class CommBase(tools.YggClass):
         self._last_header = None
         self._work_comms = {}
         self.single_use = single_use
+        self.touches_model = touches_model
         self._used = False
         self._multiple_first_send = True
         self._n_sent = 0
@@ -482,11 +483,12 @@ class CommBase(tools.YggClass):
         #     self._timeout_drain = False
         # else:
         #     self._timeout_drain = self.timeout
-        self._closing_event = threading.Event()
-        self._closing_thread = tools.YggThread(target=self.linger_close,
-                                               name=self.name + '.ClosingThread')
-        self._eof_recv = threading.Event()
-        self._eof_sent = threading.Event()
+        self._closing_event = multitasking.Event()
+        self._closing_thread = multitasking.YggTask(
+            target=self.linger_close,
+            name=self.name + '.ClosingTask')
+        self._eof_recv = multitasking.Event()
+        self._eof_sent = multitasking.Event()
         self._field_backlog = dict()
         if self.single_use:
             self._eof_recv.set()
@@ -501,6 +503,16 @@ class CommBase(tools.YggClass):
         else:
             self.open()
 
+    def __getstate__(self):
+        if self.is_open and (self._commtype != 'buffer'):  # pragma: debug
+            raise RuntimeError("Cannot pickle an open comm.")
+        return super(CommBase, self).__getstate__()
+
+    def __setstate__(self, state):
+        super(CommBase, self).__setstate__(state)
+        if self.is_interface:  # pragma: debug
+            atexit.register(self.atexit)
+
     def _init_before_open(self, **kwargs):
         r"""Initialization steps that should be performed after base class, but
         before the comm is opened."""
@@ -698,8 +710,8 @@ class CommBase(tools.YggClass):
             language = lang_list
         if isinstance(language, list):
             out = (not use_any)
-            for l in language:
-                if not cls.is_installed(language=l):
+            for lang in language:
+                if not cls.is_installed(language=lang):
                     if not use_any:
                         out = False
                         break
@@ -741,6 +753,11 @@ class CommBase(tools.YggClass):
             self._comm_class = str(name_cls).split("'")[1].split(".")[-1]
         return self._comm_class
 
+    @property
+    def model_name(self):
+        r"""str: Name of the model using the comm."""
+        return os.environ.get('YGG_MODEL_NAME', '')
+
     @classmethod
     def underlying_comm_class(self):
         r"""str: Name of underlying communication class."""
@@ -842,6 +859,7 @@ class CommBase(tools.YggClass):
 
     def open(self):
         r"""Open the connection."""
+        self.debug("Openning %s", self.address)
         self.bind()
 
     def _close(self, *args, **kwargs):
@@ -856,7 +874,7 @@ class CommBase(tools.YggClass):
                 comm. Defaults to False.
 
         """
-        self.debug('')
+        self.debug("Closing %s", self.address)
         if linger and self.is_open:
             self.linger()
         else:
@@ -893,7 +911,8 @@ class CommBase(tools.YggClass):
         if self.language_driver.comm_linger:  # pragma: matlab
             self.linger_close()
             self._closing_thread.set_terminated_flag()
-        self.debug("current_thread = %s", threading.current_thread().name)
+        self.debug("current_thread = %s",
+                   self._closing_thread.get_current_task())
         try:
             self._closing_thread.start()
             _started_thread = True
@@ -913,6 +932,8 @@ class CommBase(tools.YggClass):
         r"""Wait for messages to drain."""
         self.debug('')
         if self.direction == 'recv':
+            while self.is_open and (self.n_msg_recv_drain > 0):  # pragma: debug
+                self.recv(timeout=0)
             self.wait_for_confirm(timeout=self._timeout_drain)
         else:
             self.drain_messages(variable='n_msg_send')
@@ -1116,6 +1137,7 @@ class CommBase(tools.YggClass):
             bool: True if the object is empty, False otherwise.
 
         """
+        from yggdrasil.tests import assert_equal
         try:
             assert_equal(msg, emsg)
         except AssertionError:
@@ -1193,8 +1215,8 @@ class CommBase(tools.YggClass):
 
     def signon_to_server(self):
         r"""Add a client to an existing server or create one."""
-        with _server_lock:
-            global _registered_servers
+        global _registered_servers
+        with _registered_servers.lock:
             if self._server is None:
                 if not self.server_exists(self.address):
                     self.debug("Creating new server")
@@ -1207,7 +1229,8 @@ class CommBase(tools.YggClass):
 
     def signoff_from_server(self):
         r"""Remove a client from the server."""
-        with _server_lock:
+        global _registered_servers
+        with _registered_servers.lock:
             if self._server is not None:
                 self.debug("Signing off")
                 self._server.remove_client()
@@ -1358,6 +1381,7 @@ class CommBase(tools.YggClass):
         # Don't send metadata for files
         # kwargs.setdefault('dont_encode', self.is_file)
         kwargs.setdefault('no_metadata', self.is_file)
+        kwargs.setdefault('max_header_size', self.maxMsgSize)
         return self.serializer.serialize(*args, **kwargs)
 
     def deserialize(self, *args, **kwargs):
@@ -1454,15 +1478,28 @@ class CommBase(tools.YggClass):
         # self.remove_work_comm(workcomm.uuid, in_thread=True)
         return ret
             
-    def on_send_eof(self):
+    def on_send_eof(self, header_kwargs=None):
         r"""Actions to perform when EOF being sent.
 
+        Args:
+            header_kwargs (dict, optional): Keyword arguments that should be
+                added to the header.
+
         Returns:
             bool: True if EOF message should be sent, False otherwise.
 
         """
         self.debug('')
-        msg_s = self.eof_msg
+        if header_kwargs:
+            msg_s = self.serialize(self.eof_msg,
+                                   header_kwargs=header_kwargs)
+        else:
+            msg_s = self.eof_msg
+        msg_len = len(msg_s)
+        if (msg_len > self.maxMsgSize) and (self.maxMsgSize != 0):  # pragma: debug
+            raise NotImplementedError(("EOF message with header (%d) "
+                                       "exceeds max message size (%d).")
+                                      % (msg_len, self.maxMsgSize))
         with self._closing_thread.lock:
             if not self._eof_sent.is_set():
                 self._eof_sent.set()
@@ -1491,7 +1528,7 @@ class CommBase(tools.YggClass):
         if len(msg) == 1:
             msg = msg[0]
         if self.is_eof(msg):
-            flag, msg_s = self.on_send_eof()
+            flag, msg_s = self.on_send_eof(header_kwargs=header_kwargs)
         else:
             flag = True
             # Covert object
@@ -1597,6 +1634,7 @@ class CommBase(tools.YggClass):
         if (msg_len < self.maxMsgSize) or (self.maxMsgSize == 0):
             flag = self._safe_send(msg_s, **kwargs)
         else:
+            self.special_debug('Message will be split.')
             flag = self._safe_send(msg_s[:self.maxMsgSize])
             if flag:
                 # Send remainder of message using work comm
@@ -1605,7 +1643,7 @@ class CommBase(tools.YggClass):
             else:  # pragma: debug
                 self.special_debug("Sending message header failed.")
         if flag:
-            self.debug('Sent %d bytes', msg_len)
+            self.debug('Sent %d bytes to %s', msg_len, self.address)
         else:  # pragma: debug
             self.special_debug('Failed to send %d bytes', msg_len)
         return flag
@@ -1816,7 +1854,7 @@ class CommBase(tools.YggClass):
         else:
             msg_len = 1
         if flag and (msg_len > 0):
-            self.debug('%d bytes received', msg_len)
+            self.debug('%d bytes received from %s', msg_len, self.address)
         return flag, msg
         
     def recv_nolimit(self, *args, **kwargs):
@@ -1868,6 +1906,8 @@ class CommBase(tools.YggClass):
             RuntimeError: If the field order can not be determined.
 
         """
+        from yggdrasil.metaschema.datatypes.JSONArrayMetaschemaType import (
+            JSONArrayMetaschemaType)
         metadata = kwargs.get('header_kwargs', {})
         if 'field_order' in kwargs:
             kwargs.setdefault('key_order', kwargs.pop('field_order'))
@@ -1901,6 +1941,8 @@ class CommBase(tools.YggClass):
         Raises:
 
         """
+        from yggdrasil.metaschema.datatypes.JSONObjectMetaschemaType import (
+            JSONObjectMetaschemaType)
         if 'field_order' in kwargs:
             kwargs.setdefault('key_order', kwargs.pop('field_order'))
         key_order = kwargs.pop('key_order', None)
