@@ -261,10 +261,17 @@ int ipc_comm_send(const comm_t *x, const char *data, const size_t len) {
       ygglog_debug("ipc_comm_send(%s): msgsnd, sleep", x->name);
       usleep(YGG_SLEEP_TIME);
     } else {
-      ygglog_error("ipc_comm_send:  msgsend(%d, %p, %d, IPC_NOWAIT) ret(%d), errno(%d): %s",
-		   (int*)(x->handle), &t, len, ret, errno, strerror(errno));
-      ret = -1;
-      break;
+      struct msqid_ds buf;
+      int rtrn = msgctl(handle, IPC_STAT, &buf);
+      if ((rtrn == 0) && ((buf.msg_qnum + len) > buf.msg_qbytes)) {
+	ygglog_debug("ipc_comm_send(%s): msgsnd, queue full, sleep", x->name);
+	usleep(YGG_SLEEP_TIME);
+      } else {
+	ygglog_error("ipc_comm_send:  msgsend(%d, %p, %d, IPC_NOWAIT) ret(%d), errno(%d): %s",
+		     ((int*)(x->handle))[0], &t, len, ret, errno, strerror(errno));
+	ret = -1;
+	break;
+      }
     }
   }
   ygglog_debug("ipc_comm_send(%s): returning %d", x->name, ret);
