@@ -59,11 +59,6 @@ public:
       ygglog_throw_error("JSONArrayMetaschemaType: Items missing.");
     if (!(type_doc[item_key_].IsArray()))
       ygglog_throw_error("JSONArrayMetaschemaType: Items must be an array.");
-    if (type_doc.HasMember("format_str")) {
-      if (!(type_doc["format_str"].IsString()))
-	ygglog_throw_error("JSONArrayMetaschemaType: format_str must be a string.");
-      strncpy(format_str_, type_doc["format_str"].GetString(), 1000);
-    }
     rapidjson::SizeType i;
     MetaschemaTypeVector items;
     for (i = 0; i < type_doc[item_key_].Size(); i++) {
@@ -369,6 +364,41 @@ public:
       }
     }
   }
+  /*!
+    @brief Get the type associated with an item.
+    @param[in] index const size_t Index of item to get type for.
+    @returns MetaschemaType* Pointer to type class for item.
+   */
+  const MetaschemaType* get_item_type(const size_t index) const override {
+    MetaschemaType* out = NULL;
+    if (index >= items_.size()) {
+      ygglog_throw_error("JSONArrayMetaschemaType::get_item_type: There are %lu items, but item %lu was requested.", items_.size(), index);
+    }
+    out = items_[index];
+    return out;
+  }
+  /*!
+    @brief Set the type associated with an item.
+    An error will be raised if the property identified by key is
+    already present and the provided type does not match the existing
+    type.
+    @param[in] index const size_t Index of the item to set the type for.
+    @param[in] itemtype const MetaschemaType* Pointer to item type that
+    should be associated with the provided index.
+   */
+  void set_item_type(const size_t index, const MetaschemaType* itemtype) override {
+    if (index < items_.size()) {
+      if ((*(items_[index])) != (*itemtype)) {
+	printf("New type:\n");
+	itemtype->display();
+	printf("Existing type:\n");
+	items_[index]->display();
+	ygglog_throw_error("JSONArrayMetaschemaType::set_item_type: New type dosn't match existing type for item %ld", index);
+      }
+    } else {
+      items_.push_back(itemtype->copy());
+    }
+  }
   /*!
     @brief Update the type object with info from provided variable arguments for serialization.
     @param[in,out] nargs size_t Number of arguments contained in ap. On output
@@ -559,10 +589,6 @@ public:
    */
   bool encode_type_prop(rapidjson::Writer<rapidjson::StringBuffer> *writer) const override {
     if (!(MetaschemaType::encode_type_prop(writer))) { return false; }
-    if (strlen(format_str_) > 0) {
-      writer->Key("format_str");
-      writer->String(format_str_, (rapidjson::SizeType)strlen(format_str_));
-    }
     writer->Key(item_key_);
     writer->StartArray();
     size_t i;
