@@ -268,6 +268,10 @@ public:
       ygglog_throw_error("MetaschemaType::update: New type information is NULL.");
     }
     if (strcmp(type_, new_info->type()) != 0) {
+      printf("New type:\n");
+      new_info->display();
+      printf("Existing type:\n");
+      display();
       ygglog_throw_error("MetaschemaType::update: Cannot update type %s to type %s.",
 			 type_, new_info->type());
     }
@@ -377,6 +381,50 @@ public:
     UNUSED(force);
     ygglog_throw_error("MetaschemaType::set_length: Cannot set length for type '%s'.", type_);
   }
+  /*!
+    @brief Get the type associated with an item.
+    @param[in] index const size_t Index of item to get type for.
+    @returns MetaschemaType* Pointer to type class for item.
+   */
+  virtual const MetaschemaType* get_item_type(const size_t index) const {
+    MetaschemaType* out = NULL;
+    UNUSED(index);
+    ygglog_throw_error("MetaschemaType::get_item_type: Cannot get item type for type '%s'.", type_);
+    return out;
+  }
+  /*!
+    @brief Set the type associated with an item.
+    @param[in] index const size_t Index of the item to set the type for.
+    @param[in] itemtype const MetaschemaType* Pointer to item type that
+    should be associated with the provided index.
+   */
+  virtual void set_item_type(const size_t index, const MetaschemaType* itemtype) {
+    UNUSED(index);
+    UNUSED(itemtype);
+    ygglog_throw_error("MetaschemaType::set_item_type: Cannot set item type for type '%s'.", type_);
+  }
+  /*!
+    @brief Get the type associated with a property.
+    @param[in] key const char* Property key to get type for.
+    @returns MetaschemaType* Pointer to type class for property.
+   */
+  virtual const MetaschemaType* get_property_type(const char* key) const {
+    MetaschemaType* out = NULL;
+    UNUSED(key);
+    ygglog_throw_error("MetaschemaType::get_property_type: Cannot get property type for type '%s'.", type_);
+    return out;
+  }
+  /*!
+    @brief Set the type associated with a property.
+    @param[in] key const char* Property key to set type for.
+    @param[in] proptype const MetaschemaType* Pointer to property type
+    that should be associated with the provided key.
+   */
+  virtual void set_property_type(const char* key, const MetaschemaType* proptype) {
+    UNUSED(key);
+    UNUSED(proptype);
+    ygglog_throw_error("MetaschemaType::set_property_type: Cannot set property type for type '%s'.", type_);
+  }
   /*!
     @brief Set the _variable_length private variable.
     @param[in] new_variable_length bool New value.
@@ -1239,10 +1287,10 @@ YggGeneric::YggGeneric() : type(NULL), data(NULL), nbytes(0) {};
 
 YggGeneric::YggGeneric(const MetaschemaType* in_type, void* in_data, size_t in_nbytes) : type(NULL), data(NULL), nbytes(in_nbytes) {
   set_type(in_type);
-  set_data(in_data);
   if (nbytes == 0) {
     nbytes = type->nbytes();
   }
+  set_data(in_data);
 };
 
 YggGeneric::YggGeneric(const YggGeneric &other) :
@@ -1372,6 +1420,131 @@ void YggGeneric::get_data(char* obj, size_t nelements) const {
   get_data(obj, nelements, true);
 };
 
+void* YggGeneric::get_data(const MetaschemaType* exp_type) const {
+  if (*type != *exp_type) {
+    printf("Wrapped Type:\n");
+    type->display();
+    printf("Excepted Type:\n");
+    exp_type->display();
+    ygglog_throw_error("YggGeneric::get_data: Type of generic wrapped data does not match expected type.");
+  }
+  return get_data();
+};
+
+size_t YggGeneric::get_data_array_size() const {
+  if (type->type_code() != T_ARRAY) {
+    ygglog_throw_error("YggGeneric::get_data_array_item: Object is not an array.");
+  }
+  YggGenericVector* arr = (YggGenericVector*)get_data();
+  if (arr == NULL) {
+    ygglog_throw_error("YggGeneric::get_data_array_item: Array is NULL.");
+  }
+  return arr->size();
+};
+
+size_t YggGeneric::get_data_map_size() const {
+  if (type->type_code() != T_OBJECT) {
+    ygglog_throw_error("YggGeneric::get_data_map_size: Object is not a map.");
+  }
+  YggGenericMap* map = (YggGenericMap*)get_data();
+  if (map == NULL) {
+    ygglog_throw_error("YggGeneric::get_data_map_size: Map is NULL.");
+  }
+  return map->size();
+};
+
+size_t YggGeneric::get_data_map_keys(char*** keys) const {
+  size_t i;
+  if (type->type_code() != T_OBJECT) {
+    ygglog_throw_error("YggGeneric::get_data_map_keys: Object is not a map.");
+  }
+  YggGenericMap* map = (YggGenericMap*)get_data();
+  if (map == NULL) {
+    ygglog_throw_error("YggGeneric::get_data_map_keys: Map is NULL.");
+  }
+  keys[0] = (char**)realloc(keys[0], map->size()*sizeof(char*));
+  YggGenericMap::iterator it;
+  for (i = 0, it = map->begin(); it != map->end(); it++, i++) {
+    keys[0][i] = (char*)malloc(it->first.length() + 1);
+    // keys[0][i] = (char*)realloc(keys[0][i], it->first.length() + 1);
+    strcpy(keys[0][i], it->first.c_str());
+  }
+  return map->size();
+};
+
+void* YggGeneric::get_data_array_item(const size_t i,
+				      const MetaschemaType* item_type,
+				      bool return_generic) const {
+  if (type->type_code() != T_ARRAY) {
+    ygglog_throw_error("YggGeneric::get_data_array_item: Object is not an array.");
+  }
+  YggGenericVector* arr = (YggGenericVector*)get_data();
+  if (arr == NULL) {
+    ygglog_throw_error("YggGeneric::get_data_array_item: Array is NULL.");
+  }
+  if (i > arr->size()) {
+    ygglog_throw_error("YggGeneric::get_data_array_item: Array has %lu elements, but %lu were requested.", arr->size(), i);
+  }
+  YggGeneric* out = (*arr)[i];
+  if (return_generic) {
+    return (void*)out;
+  } else {
+    return out->get_data(item_type);
+  }
+};
+
+void YggGeneric::set_data_array_item(const size_t index,
+				     const YggGeneric* value) {
+  if (type->type_code() != T_ARRAY) {
+    ygglog_throw_error("YggGeneric::set_data_array_item: Object is not an array.");
+  }
+  YggGenericVector* arr = (YggGenericVector*)get_data();
+  if (arr == NULL) {
+    ygglog_throw_error("YggGeneric::set_data_array_item: Array is NULL.");
+  }
+  type->set_item_type(index, value->get_type());
+  if (index < arr->size()) {
+    (*arr)[index] = value->copy();
+  } else {
+    arr->push_back(value->copy());
+  }
+};
+
+void* YggGeneric::get_data_map_item(const char *key,
+				    const MetaschemaType* item_type,
+				    bool return_generic) const {
+  if (type->type_code() != T_OBJECT) {
+    ygglog_throw_error("YggGeneric::get_data_map_item: Object is not a map.");
+  }
+  YggGenericMap* map = (YggGenericMap*)get_data();
+  if (map == NULL) {
+    ygglog_throw_error("YggGeneric::get_data_map_item: Map is NULL.");
+  }
+  YggGenericMap::iterator map_it = map->find(key);
+  if (map_it == map->end()) {
+    ygglog_throw_error("YggGeneric::get_data_map_item: Could not located item for key '%s'.", key);
+  }
+  YggGeneric* out = map_it->second;
+  if (return_generic) {
+    return (void*)out;
+  } else {
+    return out->get_data(item_type);
+  }
+};
+
+void YggGeneric::set_data_map_item(const char *key,
+				   const YggGeneric* value) {
+  if (type->type_code() != T_OBJECT) {
+    ygglog_throw_error("YggGeneric::set_data_map_item: Object is not a map.");
+  }
+  YggGenericMap* map = (YggGenericMap*)get_data();
+  if (map == NULL) {
+    ygglog_throw_error("YggGeneric::set_data_map_item: Map is NULL.");
+  }
+  type->set_property_type(key, value->get_type());
+  (*map)[key] = value->copy();
+};
+
 
 typedef std::map<std::string, MetaschemaType*> MetaschemaTypeMap;
 typedef std::vector<MetaschemaType*> MetaschemaTypeVector;
