@@ -145,6 +145,8 @@ std::map<const char*, int, strcomp> get_type_map() {
     global_type_map["instance"] = T_INSTANCE;
     global_type_map["schema"] = T_SCHEMA;
     global_type_map["any"] = T_ANY;
+    // Aliases
+    global_type_map["map"] = T_OBJECT;
   }
   return global_type_map;
 };
@@ -283,6 +285,71 @@ public:
     Defaults to 1 if not provided.
    */
   void get_data(char* obj, size_t nelements) const;
+  /*!
+    @brief Get data and check against a data type.
+    @param[in] exp_type MetaschemaType* Type that is expected to be
+    returned.
+   */
+  void* get_data(const MetaschemaType* exp_type) const;
+  /*!
+    @brief Get the size of a map.
+    @returns size_t Size of the wrapped map.
+   */
+  size_t get_data_array_size() const;
+  /*!
+    @brief Get the size of an array.
+    @returns size_t Size of the wrapped array.
+   */
+  size_t get_data_map_size() const;
+  /*!
+    @brief Get an array of keys in the map.
+    @param[out] keys char*** Pointer to memory that should be reallocated
+    to allow for a storate of character array pointers.
+    @returns size_t Number of keys in keys array.
+   */
+  size_t get_data_map_keys(char*** keys) const;
+  /*!
+    @brief Get an array item.
+    @param[in] i const size_t Index of array element that should be
+    returned.
+    @param[in] item_type MetaschemaType* Type that should be returned.
+    @param[in] return_generic bool If true, the generic wrapper object
+    will be returned. Defaults to false.
+    @returns void* Pointer to the data.
+   */
+  void* get_data_array_item(const size_t i,
+			    const MetaschemaType* item_type,
+			    bool return_generic=false) const;
+  /*!
+    @brief Set an array item.
+    @param[in] index const size_t Index of array element that should
+    be set. If larger than the current size of the array, the element
+    will be appended to the end of the array.
+    @param value const YggGeneric* Pointer to value that the indexed
+    item in the array should be set to.
+   */
+  void set_data_array_item(const size_t index,
+			   const YggGeneric* value);
+  /*!
+    @brief Get a map item.
+    @param[in] key const char* String key for item that should be
+    returned.
+    @param[in] item_type MetaschemaType* Type that should be returned.
+    @param[in] return_generic bool If true, the generic wrapper object
+    will be returned. Defaults to false.
+    @returns void* Pointer to the data.
+   */
+  void* get_data_map_item(const char *key,
+			  const MetaschemaType* item_type,
+			  bool return_generic=false) const;
+  /*!
+    @brief Set a map item.
+    @param[in] key const char* String key for item that should be set.
+    @param[in] value const YggGeneric* Pointer to value that map item
+    should be set to.
+   */
+  void set_data_map_item(const char *key, const YggGeneric* value);
+  
 };
 
 
@@ -324,6 +391,9 @@ PyObject* import_python_module(const char* module_name,
   initialize_python(error_prefix);
   PyObject* out = PyImport_ImportModule(module_name);
   if (out == NULL) {
+    // if (PyErr_Occurred()) {
+    PyErr_Print();
+    //}
     ygglog_throw_error("%simport_python_module: Failed to import Python module '%s'.",
 		       error_prefix, module_name);
   }
@@ -345,6 +415,9 @@ PyObject* import_python_class(const char* module_name, const char* class_name,
   PyObject *out = PyObject_GetAttrString(py_module, class_name);
   Py_DECREF(py_module);
   if (out == NULL) {
+    //if (PyErr_Occurred()) {
+    PyErr_Print();
+    //}
     ygglog_throw_error("import_python_class: Failed to import Python class '%s'.", class_name);
   }
   return out;
@@ -416,6 +489,7 @@ void check_python_object(PyObject *pyobj, int type_code=-1,
     ygglog_throw_error("%scheck_python_object: Python object is not %s.", prefix, type_name);
   }
   if (PyErr_Occurred() != NULL) {
+    PyErr_Print();
     ygglog_throw_error("%scheck_python_object: Python error.", prefix);
   }
 };
