@@ -406,6 +406,9 @@ public:
 class YggRpcClient : public YggRpc {
 public:
 
+  /*! @brief Empty constructor for inheritance. */
+  YggRpcClient(yggRpc_t x) : YggRpc(x) {}
+  
   /*!
     @brief Constructor for YggRpcClient.
     @param[in] name constant character pointer name used for input and output
@@ -473,6 +476,29 @@ public:
 };
 
 
+/*!
+  @brief C++ interface to timestep synchronization functionality.
+  The YggTimesync class is a basic wrapper around the C yggRpc_t
+  structure and associated client-side functions from the YggInterface.h
+  header. It provides the user with C++ style access to basic RPC client
+  operations.
+ */
+class YggTimesync : public YggRpcClient {
+public:
+
+  /*!
+    @brief Constructor for YggTimesync.
+    @param[in] name constant character pointer name used for input and output
+    queues.
+    @param[in] t_units const char* Units that should be used for the
+    timestep. "" indicates no units.
+   */
+  YggTimesync(const char *name="timesync", const char *t_units="") :
+    YggRpcClient(yggTimesync(name, t_units)) {}
+  
+};
+
+
 /*!
   @brief C++ interface to yggAsciiFileOutput_t functionality.
   The YggAsciiFileOutput class is a basic wrapper around the C
