@@ -6,7 +6,7 @@ def test_get_language_dir():
     r"""Test the get_language_dir method."""
     assert_raises(ValueError, languages.get_language_dir, 'invalid')
     test_lang = tools.get_supported_lang()
-    test_skip = ['make', 'cmake', 'executable']
+    test_skip = ['make', 'cmake', 'executable', 'timesync']
     for lang in test_lang:
         if lang in test_skip:
             continue
