@@ -1,17 +1,95 @@
 import re
 import numpy as np
+import pandas as pd
 import unyt
 from yggdrasil import tools
-_ureg_unyt = unyt.UnitRegistry('mks')
 _unit_quantity = unyt.array.unyt_quantity
 _unit_array = unyt.array.unyt_array
-_ureg_unyt.add("ac", 4046.86, dimensions=unyt.dimensions.area,
-               tex_repr=r"\rm{ac}", offset=0.0, prefixable=False)
-_ureg_unyt.add("a", 100.0, dimensions=unyt.dimensions.area,
-               tex_repr=r"\rm{a}", offset=0.0, prefixable=True)
-unyt._unit_lookup_table.inv_name_alternatives["acre"] = "ac"
-unyt._unit_lookup_table.inv_name_alternatives["are"] = "a"
-unyt._unit_lookup_table.inv_name_alternatives["hectare"] = "ha"
+_ureg_unyt = None
+
+
+def get_ureg():
+    r"""Get the unit registry."""
+    global _ureg_unyt
+    if _ureg_unyt is None:
+        _ureg_unyt = unyt.UnitRegistry('mks')
+        _ureg_unyt.add("ac", 4046.86, dimensions=unyt.dimensions.area,
+                       tex_repr=r"\rm{ac}", offset=0.0, prefixable=False)
+        _ureg_unyt.add("a", 100.0, dimensions=unyt.dimensions.area,
+                       tex_repr=r"\rm{a}", offset=0.0, prefixable=True)
+        _ureg_unyt.add("j", 1.0, dimensions=unyt.dimensions.energy,
+                       tex_repr=r"\rm{J}", offset=0.0, prefixable=True)
+        # _ureg_unyt.add("cel", 1.0, dimensions=unyt.dimensions.temperature,
+        #                tex_repr=r"^\circ\rm{C}", offset=-273.15, prefixable=True)
+        # _ureg_unyt.add("j", 1.0, dimensions=unyt.dimensions.specific_flux,
+        #                tex_repr=r"\rm{Jy}", prefixable=True)
+        # _ureg_unyt.add("CH2O", 1.0, dimensions=unyt.dimensions.dimensionless,
+        #                tex_repr=r"\rm{CH2O}", offset=0.0, prefixable=False)
+        unyt._unit_lookup_table.inv_name_alternatives["acre"] = "ac"
+        unyt._unit_lookup_table.inv_name_alternatives["are"] = "a"
+        unyt._unit_lookup_table.inv_name_alternatives["hectare"] = "ha"
+    return _ureg_unyt
+
+
+def convert_to_pandas_timedelta(x):
+    r"""Convert variable with time units to a pandas.Timedelta instance.
+
+    Args:
+        x (object): Scalar/array with units to convert to a pandas.Timedelta
+            instance.
+
+    Returns:
+        pandas.Timedelta: Equivalent Timedelta variable.
+
+    """
+    assert(has_units(x))
+    t_data = get_data(x)
+    t_unit = get_units(x)
+    unit_map = {'ns': 'ns',
+                (tools.bytes2str(b'\xc2\xb5') + 's'): 'us',
+                'ms': 'ms',
+                's': 's',
+                'min': 'm',
+                'hr': 'h',
+                'day': 'D'}
+    return pd.Timedelta(t_data, unit=unit_map[t_unit])
+
+
+def convert_from_pandas_timedelta(x):
+    r"""Covert a pandas.Timedelta instance to a scalar/array with
+    time units.
+
+    Args:
+        x (pandas.Timedelta): Timedelta variable to convert.
+
+    Returns:
+        object: Equivalent scalar/array with units.
+
+    """
+    return add_units(x.total_seconds(), 's')
+
+
+def convert_matlab_unit_string(m_str):  # pragma: matlab
+    r"""Convert Matlab unit string to string that the Python package
+    can understand.
+
+    Args:
+        m_str (str): Matlab units string to convert.
+
+    Returns:
+        str: Converted string.
+
+    """
+    out = m_str
+    replacements = {'h': 'hr'}
+    regex_mu = tools.bytes2str(b'\xc2\xb5')
+    regex = r'(?P<name>[A-Za-z%s]+)' % regex_mu
+    for x in re.finditer(regex, m_str):
+        xdict = x.groupdict()
+        if xdict['name'] in replacements:
+            xdict['name'] = replacements[xdict['name']]
+            out = out[:(x.start())] + xdict['name'] + out[(x.end()):]
+    return out
 
 
 def convert_R_unit_string(r_str):
@@ -26,10 +104,13 @@ def convert_R_unit_string(r_str):
 
     """
     out = []
+    replacements = {'h': 'hr'}
     regex_mu = tools.bytes2str(b'\xc2\xb5')
     regex = r'(?P<name>[A-Za-z%s]+)(?P<exp>-?[0-9]*)(?: |$)' % regex_mu
     for x in re.finditer(regex, r_str):
         xdict = x.groupdict()
+        if xdict['name'] in replacements:
+            xdict['name'] = replacements[xdict['name']]
         if xdict['exp']:
             out.append('({name}**{exp})'.format(**xdict))
         else:
@@ -103,6 +184,7 @@ def add_units(arr, unit_str, dtype=None):
         unyt.unyt_array: Array with units.
 
     """
+    ureg = get_ureg()
     unit_str = tools.bytes2str(unit_str)
     if is_null_unit(unit_str):
         return arr
@@ -115,10 +197,10 @@ def add_units(arr, unit_str, dtype=None):
             dtype = np.array([arr]).dtype
     if isinstance(arr, np.ndarray) and (arr.ndim > 0):
         out = unyt.unyt_array(arr, unit_str, dtype=dtype,
-                              registry=_ureg_unyt)
+                              registry=ureg)
     else:
         out = unyt.unyt_quantity(arr, unit_str, dtype=dtype,
-                                 registry=_ureg_unyt)
+                                 registry=ureg)
     return out
 
 
@@ -175,7 +257,7 @@ def as_unit(ustr):
 
     """
     try:
-        out = unyt.Unit(ustr, registry=_ureg_unyt)
+        out = unyt.Unit(ustr, registry=get_ureg())
     except unyt.exceptions.UnitParseError as e:
         raise ValueError(str(e))
     return out
@@ -202,7 +284,7 @@ def is_unit(ustr):
 
 
 def convert_to(arr, new_units):
-    r"""Convert qunatity with units to new units. Objects without units
+    r"""Convert quantity with units to new units. Objects without units
     will be returned with the new units.
 
     Args:
@@ -223,3 +305,22 @@ def convert_to(arr, new_units):
     except unyt.exceptions.UnitConversionError as e:
         raise ValueError(str(e))
     return out
+
+
+def get_conversion_function(old_units, new_units):
+    r"""Get a function that will convert a scalar/array from one unit
+    to another.
+
+    Args:
+        old_units (str): Units to convert from.
+        new_units (str): Units to convert to.
+
+    Returns:
+        function: Conversion function that takes scalar/array as input
+            and returns converted scalar/array.
+
+    """
+    def fconvert(x):
+        ux = add_units(x, old_units)
+        return get_data(convert_to(ux, new_units))
+    return fconvert
